# General imports
from typing import List

# References for functions and classes in the rhdhv_fem package
from rhdhv_fem.client.fem_parser import RhdhvFemParser
from rhdhv_fem.materials import MaterialModel
from rhdhv_fem.diana_utils import diana_diagram


### HELP FUNCTION ###
def get_diana_aspect(aspects: list):
    """
    Returns a dictionary of diana aspects when a list with MatrialAspect objects is passed
    
    Input:
        - aspects(List[MaterialAspect]) - list of MaterialAspect objects
        
    Output:
        - dict - Dictionary with Diana aspects
    """
    aspect_dict = {}
    if aspects:
        for aspect in aspects:
            to_diana_dict = aspect.to_diana()
            if "ASPECT" in aspect_dict.keys():
                aspect_dict["ASPECT"] = aspect_dict['ASPECT'] + [to_diana_dict["ASPECT"]]
            else:
                aspect_dict["ASPECT"] = [to_diana_dict["ASPECT"]]
            for key, value in to_diana_dict.items():
                if key != 'ASPECT':
                    aspect_dict[key] = value
    return aspect_dict
#####################


@RhdhvFemParser(category='MaterialModels')
class UniaxialNonLinearElasticModel(LinearElasticModel):
    def __init__(self, youngs_modulus: float, stress_strain_diagram: List[List[float]]):
        """
        This is the class for the linear elastic orthotropic material models.

        Input:
            - youngs_modulus (float): The youngs modulus of the material in [N/m2].
            - stress_strain_diagram (list of list with floats): The strains and stresses of the material
        """
        self.youngs_modulus = youngs_modulus
        self.stress_strain_diagram = stress_strain_diagram
        MaterialModel.__init__(self)
    
    @property
    def is_linear(self):
        return False

    def to_diana(self, density: float):
        """
        Method of 'UniaxialNonLinearElasticModel' class to create a dictionary for DIAA to create the selected material
        model. This method can be seen as a translation between the properties in the fem package and the software.

        Input:
            - density (float): density of material.

        Output:
            - Returns a dictionary with the parameters needed for the material model in DIANA.
        """
        parameters = {
            'ELASTI/ELASTI/YOUNG': self.youngs_modulus,
            'ELASTI/EPSSIG': diana_diagram(self.stress_strain_diagram),
            'ELASTI/MASS/DENSIT': density}

        return {**parameters}

    def get_diana_material_model(self):
        """
        Getter for the material model which should be used in DIANA.
        """
        return {'MCSTEL': 'UNIAXI'}


def fem_create_uniaxial_non_linear_elastic_model(project: 'Project', youngs_modulus: float,
                                                 stress_strain_diagram: List[List[float]]):
    """
    This is the class for the linear elastic orthotropic material models.

    Input:
        - youngs_modulus (float): The youngs modulus of the material in [N/m2].
        - stress_strain_diagram (list of list with floats): The strains and stresses of the material

    Output:
        - The object for a uniaxial non linear elastic material model is created.
    """
    return project.add(UniaxialNonLinearElasticModel(youngs_modulus=youngs_modulus,
                                                     stress_strain_diagram=stress_strain_diagram))
